@@ -53,6 +53,7 @@
 
 *************************************************************************/
 
+#include "config.h"
 #include <assert.h>
 #include <errno.h>
 #include <stdlib.h>
@@ -413,6 +414,8 @@ int main(int argc, char *argv[])
         snprintf(buffer, 31, "f%d", k + 1);
         buffer[31] = '\0';
         char *p = x_strndup(buffer, 31);
+        if (unlikely(!p))
+            fatal_error("%s:%d: malloc failed", __FILE__, __LINE__);
         printf ("# %s: %s\n", p, filenames[k]);
         filenames[k] = p;
     }
@@ -440,12 +443,12 @@ int main(int argc, char *argv[])
         for (k = 0; k < numfiles; k++) {
             int size = 0;
             for (n = 0; n < nruns[k]; n++) {
-                int failed_pos
+                size_t failed_pos
                     = find_dominated_point (&points[k][dim * size], dim,
                                             cumsizes[k][n] - size, minmax);
-                if (failed_pos >= 0) {
+                if (failed_pos < SIZE_MAX) {
                     fprintf (stderr,
-                             "%s: %s: set %d: point %d is dominated.\n",
+                             "%s: %s: set %d: point %zu is dominated.\n",
                              program_invocation_short_name,
                              filenames[k], n, failed_pos);
                     check_failed = true;
